@@ -6,9 +6,9 @@ from pint import Unit
 from xarray import register_dataarray_accessor, register_dataset_accessor
 from xarray.core.dtypes import NA
 
-from . import conversion
-from .conversion import no_unit_values
-from .errors import format_error_message
+from pint_xarray import conversion
+from pint_xarray.conversion import no_unit_values
+from pint_xarray.errors import format_error_message
 
 _default = object()
 
@@ -152,10 +152,9 @@ class DatasetLocIndexer:
             raise NotImplementedError("pandas-style indexing is not supported, yet")
 
         dims = self.ds.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # convert the indexes to the indexer's units
@@ -165,11 +164,13 @@ class DatasetLocIndexer:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
-        return converted.loc[stripped_indexers]
+        stripped_indexers = conversion.strip_indexer_units(indexers)
+
+        stripped = conversion.strip_units(converted)
+        converted_units = conversion.extract_units(converted)
+        indexed = stripped.loc[stripped_indexers]
+
+        return conversion.attach_units(indexed, converted_units)
 
 
 class DataArrayLocIndexer:
@@ -183,10 +184,9 @@ class DataArrayLocIndexer:
             raise NotImplementedError("pandas-style indexing is not supported, yet")
 
         dims = self.da.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # convert the indexes to the indexer's units
@@ -196,11 +196,13 @@ class DataArrayLocIndexer:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
-        return converted.loc[stripped_indexers]
+        stripped_indexers = conversion.strip_indexer_units(indexers)
+
+        stripped = conversion.strip_units(converted)
+        converted_units = conversion.extract_units(converted)
+        indexed = stripped.loc[stripped_indexers]
+
+        return conversion.attach_units(indexed, converted_units)
 
     def __setitem__(self, indexers, values):
         if not is_dict_like(indexers):
@@ -219,10 +221,7 @@ class DataArrayLocIndexer:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in converted.items()
-        }
+        stripped_indexers = conversion.strip_indexer_units(converted)
         self.da.loc[stripped_indexers] = values
 
 
@@ -252,12 +251,6 @@ class PintDataArrayAccessor:
             the data into memory. To avoid that, consider converting
             to ``dask`` first (e.g. using ``chunk``).
 
-        .. warning::
-
-            As units in dimension coordinates are not supported until
-            ``xarray`` changes the way it implements indexes, these
-            units will be set as attributes.
-
         .. note::
             Also note that datetime units (i.e. ones that match
             ``{units} since {date}``) in unit attributes will be
@@ -648,10 +641,9 @@ class PintDataArrayAccessor:
         indexers = either_dict_or_kwargs(indexers, indexers_kwargs, "reindex")
 
         dims = self.da.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # TODO: handle tolerance
@@ -659,20 +651,19 @@ class PintDataArrayAccessor:
 
         # convert the indexes to the indexer's units
         converted = conversion.convert_units(self.da, indexer_units)
+        converted_units = conversion.extract_units(converted)
+        stripped = conversion.strip_units(converted)
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
-        indexed = converted.reindex(
+        stripped_indexers = conversion.strip_indexer_units(indexers)
+        indexed = stripped.reindex(
             stripped_indexers,
             method=method,
             tolerance=tolerance,
             copy=copy,
             fill_value=fill_value,
         )
-        return indexed
+        return conversion.attach_units(indexed, converted_units)
 
     def reindex_like(
         self, other, method=None, tolerance=None, copy=True, fill_value=NA
@@ -692,19 +683,24 @@ class PintDataArrayAccessor:
         xarray.DataArray.pint.reindex
         xarray.DataArray.reindex_like
         """
-        indexer_units = conversion.extract_unit_attributes(other)
+        indexer_units = conversion.extract_units(other)
+
+        converted = conversion.convert_units(self.da, indexer_units)
+        units = conversion.extract_units(converted)
+        stripped = conversion.strip_units(converted)
+        stripped_other = conversion.strip_units(other)
 
         # TODO: handle tolerance
         # TODO: handle fill_value
 
-        converted = conversion.convert_units(self.da, indexer_units)
-        return converted.reindex_like(
-            other,
+        reindexed = stripped.reindex_like(
+            stripped_other,
             method=method,
             tolerance=tolerance,
             copy=copy,
             fill_value=fill_value,
         )
+        return conversion.attach_units(reindexed, units)
 
     def interp(
         self,
@@ -720,7 +716,8 @@ class PintDataArrayAccessor:
         converted to the units of the indexers first.
 
         .. note::
-            ``kwargs`` is passed unmodified to ``DataArray.interp``
+            ``method``, ``assume_sorted`` and ``kwargs`` are passed unmodified to
+            ``DataArray.interp``.
 
         See Also
         --------
@@ -731,10 +728,9 @@ class PintDataArrayAccessor:
         indexers = either_dict_or_kwargs(coords, coords_kwargs, "interp")
 
         dims = self.da.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # convert the indexes to the indexer's units
@@ -743,15 +739,12 @@ class PintDataArrayAccessor:
         stripped = conversion.strip_units(converted)
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
+        stripped_indexers = conversion.strip_indexer_units(indexers)
         interpolated = stripped.interp(
             stripped_indexers,
             method=method,
-            assume_sorted=False,
-            kwargs=None,
+            assume_sorted=assume_sorted,
+            kwargs=kwargs,
         )
         return conversion.attach_units(interpolated, units)
 
@@ -762,7 +755,8 @@ class PintDataArrayAccessor:
         to the units of the indexers first.
 
         .. note::
-            ``kwargs`` is passed unmodified to ``DataArray.interp``
+            ``method``, ``assume_sorted`` and ``kwargs`` are passed unmodified to
+            ``DataArray.interp``.
 
         See Also
         --------
@@ -770,13 +764,14 @@ class PintDataArrayAccessor:
         xarray.DataArray.pint.interp
         xarray.DataArray.interp_like
         """
-        indexer_units = conversion.extract_unit_attributes(other)
+        indexer_units = conversion.extract_units(other)
 
         converted = conversion.convert_units(self.da, indexer_units)
         units = conversion.extract_units(converted)
         stripped = conversion.strip_units(converted)
+        stripped_other = conversion.strip_units(other)
         interpolated = stripped.interp_like(
-            other,
+            stripped_other,
             method=method,
             assume_sorted=assume_sorted,
             kwargs=kwargs,
@@ -804,10 +799,9 @@ class PintDataArrayAccessor:
         indexers = either_dict_or_kwargs(indexers, indexers_kwargs, "sel")
 
         dims = self.da.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # TODO: handle tolerance
@@ -819,18 +813,18 @@ class PintDataArrayAccessor:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
-        indexed = converted.sel(
+        stripped_indexers = conversion.strip_indexer_units(indexers)
+
+        stripped = conversion.strip_units(converted)
+        converted_units = conversion.extract_units(converted)
+        indexed = stripped.sel(
             stripped_indexers,
             method=method,
             tolerance=tolerance,
             drop=drop,
         )
 
-        return indexed
+        return conversion.attach_units(indexed, converted_units)
 
     @property
     def loc(self):
@@ -872,10 +866,7 @@ class PintDataArrayAccessor:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in converted_indexers.items()
-        }
+        stripped_indexers = conversion.strip_indexer_units(converted_indexers)
         indexed = self.da.drop_sel(
             stripped_indexers,
             errors=errors,
@@ -984,12 +975,6 @@ class PintDatasetAccessor:
             the data into memory. To avoid that, consider converting
             to ``dask`` first (e.g. using ``chunk``).
 
-        .. warning::
-
-            As units in dimension coordinates are not supported until
-            ``xarray`` changes the way it implements indexes, these
-            units will be set as attributes.
-
         .. note::
             Also note that datetime units (i.e. ones that match
             ``{units} since {date}``) in unit attributes will be
@@ -1425,10 +1410,9 @@ class PintDatasetAccessor:
         indexers = either_dict_or_kwargs(indexers, indexers_kwargs, "reindex")
 
         dims = self.ds.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # TODO: handle tolerance
@@ -1436,20 +1420,19 @@ class PintDatasetAccessor:
 
         # convert the indexes to the indexer's units
         converted = conversion.convert_units(self.ds, indexer_units)
+        converted_units = conversion.extract_units(converted)
+        stripped = conversion.strip_units(converted)
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
-        indexed = converted.reindex(
+        stripped_indexers = conversion.strip_indexer_units(indexers)
+        indexed = stripped.reindex(
             stripped_indexers,
             method=method,
             tolerance=tolerance,
             copy=copy,
             fill_value=fill_value,
         )
-        return indexed
+        return conversion.attach_units(indexed, converted_units)
 
     def reindex_like(
         self, other, method=None, tolerance=None, copy=True, fill_value=NA
@@ -1469,19 +1452,24 @@ class PintDatasetAccessor:
         xarray.Dataset.pint.reindex
         xarray.Dataset.reindex_like
         """
-        indexer_units = conversion.extract_unit_attributes(other)
+        indexer_units = conversion.extract_units(other)
+
+        converted = conversion.convert_units(self.ds, indexer_units)
+        units = conversion.extract_units(converted)
+        stripped = conversion.strip_units(converted)
+        stripped_other = conversion.strip_units(other)
 
         # TODO: handle tolerance
         # TODO: handle fill_value
 
-        converted = conversion.convert_units(self.ds, indexer_units)
-        return converted.reindex_like(
-            other,
+        reindexed = stripped.reindex_like(
+            stripped_other,
             method=method,
             tolerance=tolerance,
             copy=copy,
             fill_value=fill_value,
         )
+        return conversion.attach_units(reindexed, units)
 
     def interp(
         self,
@@ -1497,7 +1485,8 @@ class PintDatasetAccessor:
         to the units of the indexers first.
 
         .. note::
-            ``kwargs`` is passed unmodified to ``Dataset.interp``
+            ``method``, ``assume_sorted`` and ``kwargs`` are passed unmodified to
+            ``DataArray.interp``.
 
         See Also
         --------
@@ -1508,10 +1497,9 @@ class PintDatasetAccessor:
         indexers = either_dict_or_kwargs(coords, coords_kwargs, "interp")
 
         dims = self.ds.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # convert the indexes to the indexer's units
@@ -1520,15 +1508,12 @@ class PintDatasetAccessor:
         stripped = conversion.strip_units(converted)
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
+        stripped_indexers = conversion.strip_indexer_units(indexers)
         interpolated = stripped.interp(
             stripped_indexers,
             method=method,
-            assume_sorted=False,
-            kwargs=None,
+            assume_sorted=assume_sorted,
+            kwargs=kwargs,
         )
         return conversion.attach_units(interpolated, units)
 
@@ -1539,7 +1524,8 @@ class PintDatasetAccessor:
         converted to the units of the indexers first.
 
         .. note::
-            ``kwargs`` is passed unmodified to ``Dataset.interp``
+            ``method``, ``assume_sorted`` and ``kwargs`` are passed unmodified to
+            ``DataArray.interp``.
 
         See Also
         --------
@@ -1547,13 +1533,14 @@ class PintDatasetAccessor:
         xarray.Dataset.pint.interp
         xarray.Dataset.interp_like
         """
-        indexer_units = conversion.extract_unit_attributes(other)
+        indexer_units = conversion.extract_units(other)
 
         converted = conversion.convert_units(self.ds, indexer_units)
         units = conversion.extract_units(converted)
         stripped = conversion.strip_units(converted)
+        stripped_other = conversion.strip_units(other)
         interpolated = stripped.interp_like(
-            other,
+            stripped_other,
             method=method,
             assume_sorted=assume_sorted,
             kwargs=kwargs,
@@ -1581,10 +1568,9 @@ class PintDatasetAccessor:
         indexers = either_dict_or_kwargs(indexers, indexers_kwargs, "sel")
 
         dims = self.ds.dims
+        indexer_units = conversion.extract_indexer_units(indexers)
         indexer_units = {
-            name: conversion.extract_indexer_units(indexer)
-            for name, indexer in indexers.items()
-            if name in dims
+            name: indexer for name, indexer in indexer_units.items() if name in dims
         }
 
         # TODO: handle tolerance
@@ -1596,18 +1582,18 @@ class PintDatasetAccessor:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in indexers.items()
-        }
-        indexed = converted.sel(
+        stripped_indexers = conversion.strip_indexer_units(indexers)
+
+        stripped = conversion.strip_units(converted)
+        converted_units = conversion.extract_units(converted)
+        indexed = stripped.sel(
             stripped_indexers,
             method=method,
             tolerance=tolerance,
             drop=drop,
         )
 
-        return indexed
+        return conversion.attach_units(indexed, converted_units)
 
     @property
     def loc(self):
@@ -1651,10 +1637,7 @@ class PintDatasetAccessor:
             raise KeyError(*e.args) from e
 
         # index
-        stripped_indexers = {
-            name: conversion.strip_indexer_units(indexer)
-            for name, indexer in converted_indexers.items()
-        }
+        stripped_indexers = conversion.strip_indexer_units(converted_indexers)
         indexed = self.ds.drop_sel(
             stripped_indexers,
             errors=errors,
